@@ -269,6 +269,18 @@ static char sccsid[] = "@(#)syslogd.c	5.27 (Berkeley) 10/10/88";
  *	when syslogd starts up.
  *
  *	Minor code cleanups.
+ *
+ * Tue May 14 00:03:35 MET DST 1996:  Martin Schulze
+ *	Corrected a mistake that causes the syslogd to stop logging at
+ *	some virtual consoles under Linux. This was causes by checking
+ *	the wrong error code. Thanks to Michael Nonweiler
+ *	<mrn20@hermes.cam.ac.uk> for sending me a patch.
+ *
+ * Tue May 28 00:58:45 MET DST 1996:  Martin Schulze
+ *	Corrected behaviour of blocking pipes - i.e. the whole system
+ *	hung.  Michael Nonweiler <mrn20@hermes.cam.ac.uk> has sent us
+ *	a patch to correct this.  A new logfile type F_PIPE has been
+ *	introduced.
  */
 
 
@@ -473,10 +485,11 @@ int	repeatinterval[] = { 30, 60 };	/* # of secs before flush */
 #define F_WALL		6		/* everyone logged on */
 #define F_FORW_SUSP	7		/* suspended host forwarding */
 #define F_FORW_UNKN	8		/* unknown host forwarding */
-char	*TypeNames[9] = {
+#define F_PIPE		9		/* named pipe */
+char	*TypeNames[] = {
 	"UNUSED",	"FILE",		"TTY",		"CONSOLE",
 	"FORW",		"USERS",	"WALL",		"FORW(SUSPENDED)",
-	"FORW(UNKNOWN)"
+	"FORW(UNKNOWN)", "PIPE"
 };
 
 struct	filed *Files = (struct filed *) 0;
@@ -1497,9 +1510,10 @@ void fprintlog(f, from, flags, msg)
 
 	case F_TTY:
 	case F_FILE:
+	case F_PIPE:
 		f->f_time = now;
 		dprintf(" %s\n", f->f_un.f_fname);
-		if (f->f_type != F_FILE) {
+		if (f->f_type == F_TTY || f->f_type == F_CONSOLE) {
 			v->iov_base = "\r\n";
 			v->iov_len = 2;
 		} else {
@@ -1509,11 +1523,23 @@ void fprintlog(f, from, flags, msg)
 	again:
 		if (writev(f->f_file, iov, 6) < 0) {
 			int e = errno;
+
+			/* If a named pipe is full, just ignore it for now
+			   - mrn 24 May 96 */
+			if (f->f_type == F_PIPE && e == EAGAIN)
+				break;
+
 			(void) close(f->f_file);
 			/*
 			 * Check for EBADF on TTY's due to vhangup() XXX
+			 * Linux uses EIO instead (mrn 12 May 96)
 			 */
-			if (e == EBADF && f->f_type != F_FILE) {
+			if ((f->f_type == F_TTY || f->f_type == F_CONSOLE)
+#ifdef linux
+				&& e == EIO) {
+#else
+				&& e == EBADF) {
+#endif
 				f->f_file = open(f->f_un.f_fname, O_WRONLY|O_APPEND|O_NOCTTY);
 				if (f->f_file < 0) {
 					f->f_type = F_UNUSED;
@@ -1846,6 +1872,7 @@ void init()
 	/*
 	 *  Close all open log files.
 	 */
+/*
 	Initialized = 0;
 	if ( nlogs > -1 )
 	{
@@ -1854,6 +1881,7 @@ void init()
 		free((void *) Files);
 		Files = (struct filed *) 0;
 	}
+*/
 	
 #ifdef SYSV
 	for (lognum = 0; lognum <= nlogs; lognum++ ) {
@@ -1867,6 +1895,7 @@ void init()
 
 		switch (f->f_type) {
 		  case F_FILE:
+		  case F_PIPE:
 		  case F_TTY:
 		  case F_CONSOLE:
 			(void) close(f->f_file);
@@ -1949,6 +1978,7 @@ void init()
 				printf("%s: ", TypeNames[f->f_type]);
 				switch (f->f_type) {
 				case F_FILE:
+				case F_PIPE:
 				case F_TTY:
 				case F_CONSOLE:
 					printf("%s", f->f_un.f_fname);
@@ -1971,11 +2001,11 @@ void init()
 	}
 
 	if ( AcceptRemote )
-		logmsg(LOG_SYSLOG|LOG_INFO, "syslogd " VERSION "-" PATCHLEVEL \
+		logmsg(LOG_SYSLOG|LOG_INFO, "syslogd " VERSION "-" PATCHLEVEL "#" DEBRELEASE \
 		       ": restart (remote reception)." , LocalHostName, \
 		       	ADDDATE);
 	else
-		logmsg(LOG_SYSLOG|LOG_INFO, "syslogd " VERSION "-" PATCHLEVEL \
+		logmsg(LOG_SYSLOG|LOG_INFO, "syslogd " VERSION "-" PATCHLEVEL "#" DEBRELEASE \
 		       ": restart." , LocalHostName, ADDDATE);
 	(void) signal(SIGHUP, sighup_handler);
 	dprintf("syslogd: restarted.\n");
@@ -2182,11 +2212,14 @@ void cfline(line, f)
 		dprintf ("filename: %s\n", p);	/*ASP*/
 		if (syncfile)
 			f->f_flags |= SYNC_FILE;
-		if ( *p == '|' )
-			f->f_file = open(++p, O_RDWR);
-	        else
+		if ( *p == '|' ) {
+			f->f_file = open(++p, O_RDWR|O_NONBLOCK);
+			f->f_type = F_PIPE;
+	        } else {
 			f->f_file = open(p, O_WRONLY|O_APPEND|O_CREAT|O_NOCTTY,
 					 0644);
+			f->f_type = F_FILE;
+		}
 		        
 	  	if ( f->f_file < 0 ){
 			f->f_file = F_UNUSED;
@@ -2198,8 +2231,6 @@ void cfline(line, f)
 			f->f_type = F_TTY;
 			untty();
 		}
-		else
-			f->f_type = F_FILE;
 		if (strcmp(p, ctty) == 0)
 			f->f_type = F_CONSOLE;
 		break;
